
import java.util.Date;

public class Decoder
{
  private int theYear;
  private int theMonth;
	
  public Decoder(int year, int month) {
    theYear = year;
    theMonth = month;
  }
	
  public Show decode(int code) {
    int afterMult = NoMod.multLoop(new NoMod(code), NoMod.DecodeConst).toInt();
    int bottom = afterMult % 1000;
    int top = afterMult / 1000;
    int day = ((bottom-1)/32)+1;
    int[] offsetLoopOutput = offsetLoop(day, theYear, top);
    int offset = offsetLoopOutput[0];
    int mappedTop = offsetLoopOutput[1];
    int val1 = ((bottom-1)%32 + day*(theMonth+1) + offset) % 32;
    int[] sliceOutput = sliceAndDice(val1, mappedTop);
    int channel = sliceOutput[0]+1;
    int tableIndex = sliceOutput[1];
    return new Show(theYear, theMonth, day, tableIndex, channel);
  }	
	
  private int[] offsetLoop(int day, int year, int top) {
    NoMod nm = new NoMod(top);
    int digits = nm.numSignificantDigits();
    int off = nm.sumOfDigits();
    do {
      for(int i=0; i<=(year%16); ++i){
	off += (mapTop(day, i, top, digits)%10);
      }
      top = mapTop(day, year, top, digits);
    }while(new NoMod(top).numSignificantDigits() != digits);
    off %= 32;
    int[] ret = new int[2];
    ret[0] = off;
    ret[1] = top;
    return ret;
  }
	
  private int mapTop(int day, int year, int top, int digits) {
    year %= 16;
    int d0 = top%10;
    int d1 = (top/10)%10;
    int d2 = (top/100)%10;
    int f0 = 1;
    int f1 = (year+1)%10;
    int f2 = ((year+1)*(year+2)/2)%10;
    int f3 = ((year+1)*(year+2)*(year+3)/6)%10;
    switch(digits){
    case 0:
      return 0;
    case 1:
      return (d0*f0 + day*f1) % 10;
    case 2:
      return ((d0*f0 + d1*f1 + day*f2)%10) 
+ 10 * ((d1*f0 + day*f1)%10);
    case 3:
      return ((d0*f0 + d1*f1 + d2*f2 + day*f3)%10)
+ 10 * ((d1*f0 + d2*f1 + day*f2)%10)
+ 100 * ((d2*f0 + day*f1)%10);
    default:
      throw new IndexOutOfBoundsException();
    }
  }
	
  private int[] sliceAndDice(int val1, int mappedTop) {
    MyBitSet bs1 = new MyBitSet(val1);
    MyBitSet bs2 = new MyBitSet(mappedTop);
    MyBitSet chan = new MyBitSet();
    MyBitSet out = new MyBitSet();
    if(bs1.get(0))    out.set(0);
    if(bs1.get(1))    chan.set(0);
    if(bs1.get(2))    out.set(1);
    if(bs1.get(3))    chan.set(1);
    if(bs1.get(4))    out.set(2);
    if(bs2.get(0))    out.set(3);
    if(bs2.get(1))    chan.set(2);
    if(bs2.get(2))    chan.set(3);
    if(bs2.get(3))    out.set(4);
    if(bs2.get(4))    out.set(5);
    if(bs2.get(5))    out.set(6);
    if(bs2.get(6))    chan.set(4);
    if(bs2.get(7))    out.set(7);
    if(bs2.get(8))    chan.set(5);
    if(bs2.get(9))    out.set(8);
		
    int[] ret = new int[2];
    ret[0] = chan.toInt();
    ret[1] = out.toInt();
    return ret;
  }
	
  private int[] tableLookup(int x) {
    int durationCode = x / 48;
    int startCode = x % 48;
    int[] ret = new int[2];
    ret[0] = startCode * 30;
    ret[1] = (1+durationCode) * 30;
    return ret;
  }
	
  public static void main(String[] argv) {
    Decoder dec = new Decoder(91, 5);
    Show show = dec.decode(3316);
    System.out.println(show);
		
    dec = new Decoder(92, 3);
    show = dec.decode(21362);
    System.out.println(show);
		
    try{
      Thread.sleep(10000);
    }catch(InterruptedException x){
      System.out.println("Interrupted.  Bye.");
    }
  }
}
