
public class NoMod
{
  public static int NumDigits = 8;
  public static NoMod EncodeConst = new NoMod(9371);
  public static NoMod DecodeConst = new NoMod(68150631);
	
  private int[] x;
	
  private NoMod() {
    x = new int[NumDigits];
  }
	
  public NoMod(NoMod nm) {
    this();
    for(int i=0; i<NumDigits; ++i)    x[i] = nm.x[i];
  }
	
  public NoMod(int val) {
    this();
    for(int i=0; i<NumDigits; ++i){
      x[i] = val%10;
      val /= 10;
    }
  }
	
  public void shiftLeft(int n) {
    if(n > 0){
      for(int i=NumDigits-1; i>=0; --i){
	int j = i-n;
	if(j >= 0)    x[i] = x[j];
	else    x[i] = 0;
      }
    }else{
      for(int i=0; i<NumDigits; ++i){
	int j = i+n;
	if(j < NumDigits)    x[i] = x[j];
	else    x[i] = 0;
      }
    }
  }
	
  public int numSignificantDigits() {
    for(int i=NumDigits-1; i>=0; --i){
      if(x[i] != 0)    return i+1;
    }
    return 0;
  }
	
  public int sumOfDigits() {
    int ret = 0;
    for(int i=0; i<NumDigits; ++i)    ret += x[i];
    return ret;
  }
	
  public void truncate(int digits) {
    if(digits < 0)    throw new ArrayIndexOutOfBoundsException();
    for(int i=NumDigits-1; i>=digits; --i)    x[i] = 0;
  }
	
  public boolean equals(Object obj) {
    if(obj instanceof NoMod){
      NoMod b = (NoMod)obj;
      for(int i=0; i<NumDigits; ++i){
	if(x[i] != b.x[i])    return false;
      }
      return true;
    }
    return false;
  }
	
  public boolean equals(int i) {
    return equals(new NoMod(i));
  }
	
  public String toString() {
    return new Integer(toInt()).toString();
  }
	
  public int toInt() {
    int ret = 0;
    int pow = 1;
		
    for(int i=0; i<NumDigits; ++i,pow*=10){
      ret += (pow*x[i]);
    }
    return ret;
  }
		
  public static NoMod add(NoMod a, NoMod b) {
    NoMod ret = new NoMod();
    for(int i=0; i<NumDigits; ++i){
      ret.x[i] = (a.x[i] + b.x[i])%10;
    }
    return ret;
  }
	
  public static NoMod mult(NoMod a, NoMod b) {
    NoMod ret = new NoMod();
    for(int i=0; i<NumDigits; ++i){
      int digit = a.x[i];
      if(digit != 0){
	NoMod bb = new NoMod(b);
	for(int j=0; j<NumDigits; ++j){
	  bb.x[j] = (digit * bb.x[j]) % 10;
	}
	bb.shiftLeft(i);
	ret = NoMod.add(ret, bb);
      }
    }
    return ret;
  }
	
  public static NoMod multLoop(NoMod a, NoMod b) {
    int digits = a.numSignificantDigits();
    NoMod ret = a;
    do {
      ret = mult(ret, b);
      ret.truncate(digits);
    } while(ret.numSignificantDigits() != digits);
    return ret;
  }
	
  public static void main(String[] argv) {
    NoMod a = new NoMod(123);
    NoMod b = new NoMod(456);
    System.out.println(mult(a,b).equals(43878) ? "ok" : "d'oh");
    System.out.println(multLoop(new NoMod(3316), DecodeConst).equals(8296) ? "ok" : "d'oh");
    System.out.println(multLoop(new NoMod(21362), DecodeConst).equals(36322) ? "ok" : "d'oh");
    try{
      Thread.sleep(5000);
    }catch(InterruptedException x){
      System.out.println("Caught InterruptedException");
    }
  }
}
